SystemOrganization addCategory: #'Announcements-Core'!

Set subclass: #AnnouncementSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Announcements-Core'!

!AnnouncementSet methodsFor: 'adding' stamp: 'lr 6/13/2006 08:13'!
, anAnnouncementClass
	self add: anAnnouncementClass! !

!AnnouncementSet methodsFor: 'testing' stamp: 'lr 10/3/2006 14:31'!
handles: anAnnouncementClass
	^ self anySatisfy: [ :each | each handles: anAnnouncementClass ]! !

Object subclass: #Announcement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Announcements-Core'!

!Announcement commentStamp: 'Tbn 11/12/2010 10:57' prior: 0!
This class is the superclass for events that someone might want to announce, such as a button click or an attribute change. Typically you create subclasses for your own events you want to announce.

 
!

!Announcement class methodsFor: 'public' stamp: 'lr 9/20/2006 08:18'!
, anAnnouncementClass
	^ AnnouncementSet with: self with: anAnnouncementClass! !

!Announcement class methodsFor: 'converting' stamp: 'lr 10/3/2006 14:31'!
asAnnouncement
	^ self new! !

!Announcement class methodsFor: 'testing' stamp: 'IgorStasenko 3/12/2011 17:05'!
handles: anAnnouncementClass
	^ anAnnouncementClass == self or: [ anAnnouncementClass inheritsFrom: self ]! !

!Announcement methodsFor: 'converting' stamp: 'lr 10/3/2006 14:32'!
asAnnouncement
	^ self! !

Object subclass: #AnnouncementSubscription
	instanceVariableNames: 'announcer announcementClass subscriber action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Announcements-Core'!

!AnnouncementSubscription commentStamp: 'IgorStasenko 3/12/2011 20:23' prior: 0!
The subscription is a single entry in a SubscriptionRegistry.
Several subscriptions by the same object is possible.

I know how to make myself weak or strong, only use this capability if it can't be determined at subscribe time though, as it uses become: (for thread-safety), which is quite slow.!

!AnnouncementSubscription methodsFor: 'accessing' stamp: 'IgorStasenko 3/12/2011 15:15'!
action

	^ action! !

!AnnouncementSubscription methodsFor: 'accessing' stamp: 'IgorStasenko 3/12/2011 15:15'!
action: anObject

	action := anObject! !

!AnnouncementSubscription methodsFor: 'accessing' stamp: 'IgorStasenko 3/12/2011 15:18'!
announcementClass

	^ announcementClass! !

!AnnouncementSubscription methodsFor: 'accessing' stamp: 'IgorStasenko 3/12/2011 15:18'!
announcementClass: anObject

	announcementClass := anObject! !

!AnnouncementSubscription methodsFor: 'accessing' stamp: 'IgorStasenko 3/12/2011 15:15'!
announcer

	^ announcer! !

!AnnouncementSubscription methodsFor: 'accessing' stamp: 'IgorStasenko 3/12/2011 15:13'!
announcer: anAnnouncer
	announcer := anAnnouncer! !

!AnnouncementSubscription methodsFor: 'announcing' stamp: 'IgorStasenko 3/12/2011 19:34'!
deliver: anAnnouncement

	^ (self handles: anAnnouncement class ) ifTrue: [
		action cull: anAnnouncement cull: announcer
		]! !

!AnnouncementSubscription methodsFor: 'testing' stamp: 'IgorStasenko 3/12/2011 15:29'!
handles: anAnnouncementClass

	^ announcementClass handles: anAnnouncementClass! !

!AnnouncementSubscription methodsFor: 'converting' stamp: 'IgorStasenko 3/12/2011 17:37'!
makeStrong
	" i am already strong. Do nothing "! !

!AnnouncementSubscription methodsFor: 'converting' stamp: 'IgorStasenko 3/22/2011 15:41'!
makeWeak
	
	action isBlock ifTrue: [
		self error: 'Not currently available due to missing ephemerons support'
		].

	^ announcer replace: self with: 
		(WeakAnnouncementSubscription new
			announcer: announcer;
			action: action asWeakMessageSend;
			subscriber: subscriber;
			announcementClass: announcementClass)! !

!AnnouncementSubscription methodsFor: 'accessing' stamp: 'IgorStasenko 3/12/2011 15:14'!
subscriber
	^ subscriber! !

!AnnouncementSubscription methodsFor: 'accessing' stamp: 'IgorStasenko 3/12/2011 15:14'!
subscriber: aSubscriber
	subscriber := aSubscriber! !

!AnnouncementSubscription methodsFor: 'accessing' stamp: 'IgorStasenko 3/12/2011 15:16'!
valuable: aValuable
	"Used when subscriber should be extracted from valuable object"
	self action:  aValuable.
	self subscriber: aValuable receiver.! !

Object subclass: #Announcer
	instanceVariableNames: 'registry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Announcements-Core'!

!Announcer commentStamp: 'IgorStasenko 3/12/2011 18:45' prior: 0!
The code is based on the announcements as described by Vassili Bykov in <http://www.cincomsmalltalk.com/userblogs/vbykov/blogView?searchCategory=Announcements%20Framework>. The implementation uses a threadsafe subscription registry, in the sense that registering, unregistering, and announcing from an announcer at the same time in different threads should never cause failures.

For security reasons, registry is kept private, and has no accessor like in other implementations!

!Announcer methodsFor: 'announce' stamp: 'IgorStasenko 3/12/2011 19:35'!
announce: anAnnouncement
	| announcement |
	
	announcement := anAnnouncement asAnnouncement.
	
	registry ifNotNil: [
		registry deliver: announcement
		].
	
	^ announcement! !

!Announcer methodsFor: 'private' stamp: 'IgorStasenko 3/12/2011 20:30'!
basicSubscribe: subscription
	^ registry add: subscription! !

!Announcer methodsFor: 'initialize-release' stamp: 'IgorStasenko 3/12/2011 16:50'!
initialize
	super initialize.
	registry := SubscriptionRegistry new.! !

!Announcer methodsFor: 'statistics' stamp: 'IgorStasenko 3/12/2011 21:02'!
numberOfSubscriptions
	^ registry numberOfSubscriptions
! !

!Announcer methodsFor: 'convenience' stamp: 'lr 10/27/2006 14:26'!
on: anAnnouncementClass do: aValuable
	^ self subscribe: anAnnouncementClass do: aValuable! !

!Announcer methodsFor: 'convenience' stamp: 'lr 10/27/2006 14:27'!
on: anAnnouncementClass send: aSelector to: anObject
	^ self subscribe: anAnnouncementClass send: aSelector to: anObject! !

!Announcer methodsFor: 'subscription' stamp: 'IgorStasenko 3/12/2011 18:33'!
removeSubscription: subscription
	^ registry
		remove: subscription
! !

!Announcer methodsFor: 'private' stamp: 'IgorStasenko 3/22/2011 15:27'!
replace: subscription with: newOne
	^ registry replace: subscription with: newOne
! !

!Announcer methodsFor: 'subscription' stamp: 'IgorStasenko 3/12/2011 16:52'!
subscribe: anAnnouncementClass do: aValuable 
	
	^ registry add: (
		AnnouncementSubscription new 
			announcer: self;
			announcementClass: anAnnouncementClass;
			valuable: aValuable)! !

!Announcer methodsFor: 'subscription' stamp: 'IgorStasenko 3/12/2011 20:03'!
subscribe: anAnnouncementClass send: aSelector to: anObject
	^ self subscribe: anAnnouncementClass do: (MessageSend receiver: anObject selector: aSelector)! !

!Announcer methodsFor: 'subscription' stamp: 'IgorStasenko 3/12/2011 16:30'!
unsubscribe: anObject
	registry removeSubscriber: anObject! !

!Announcer methodsFor: 'weak' stamp: 'IgorStasenko 3/12/2011 20:26'!
weak
	"announcer weak subscribe: foo"
	^ WeakSubscriptionBuilder on: self! !

!Announcer methodsFor: 'convenience' stamp: 'tg 2/25/2009 12:05'!
when: anAnnouncementClass do: aValuable
	^ self subscribe: anAnnouncementClass do: aValuable! !

Object subclass: #SubscriptionRegistry
	instanceVariableNames: 'subscriptions monitor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Announcements-Core'!

!SubscriptionRegistry commentStamp: 'IgorStasenko 3/12/2011 18:43' prior: 0!
The subscription registry is a threadsafe storage for the subscriptions to an Announcer.

In Pharo, subscriptionsFor: protocol is not implemented.
This is because Announcer does not provide public access to its registery for encapsulation reasons.
(We do not want access to the announcer from action blocks to break encapsulation to other subscribers)!

!SubscriptionRegistry methodsFor: 'add/remove' stamp: 'IgorStasenko 3/12/2011 17:28'!
add: subscription
	^ self protected: [
		subscriptions add: subscription  ]
! !

!SubscriptionRegistry methodsFor: 'announcing' stamp: 'IgorStasenko 3/12/2011 19:34'!
deliver: anAnnouncement
	| cls copy |
	cls := anAnnouncement class.
	
	"doing a copy, so subscribers can unsubscribe from announcer "
	self protected: [ copy := subscriptions copy ].
	
	copy do: [:each | each deliver: anAnnouncement ]! !

!SubscriptionRegistry methodsFor: 'initialize-release' stamp: 'IgorStasenko 3/12/2011 17:26'!
initialize
	monitor := Monitor new.
	self reset
! !

!SubscriptionRegistry methodsFor: 'accessing' stamp: 'IgorStasenko 3/12/2011 21:03'!
numberOfSubscriptions
	^ subscriptions size! !

!SubscriptionRegistry methodsFor: 'private' stamp: 'IgorStasenko 3/12/2011 17:28'!
protected: aBlock

	^ (monitor ifNil: [ monitor := Monitor new ])
		critical: [ aBlock value]! !

!SubscriptionRegistry methodsFor: 'add/remove' stamp: 'IgorStasenko 3/12/2011 17:29'!
remove: subscription
	^ self protected: [
		subscriptions remove: subscription ifAbsent: nil ]

! !

!SubscriptionRegistry methodsFor: 'add/remove' stamp: 'IgorStasenko 3/12/2011 17:29'!
removeSubscriber: subscriber
	
	^ self protected: [
		subscriptions removeAllSuchThat: [:subscription | subscription subscriber == subscriber ]]

! !

!SubscriptionRegistry methodsFor: 'add/remove' stamp: 'IgorStasenko 3/22/2011 15:42'!
replace: subscription with: newOne

	" Note that it will signal an error if subscription is not there "
	self protected: [
		subscriptions remove: subscription.
		subscriptions add: newOne 
	].
	^ newOne! !

!SubscriptionRegistry methodsFor: 'initialize-release' stamp: 'IgorStasenko 3/12/2011 21:29'!
reset
	"subscriber -> subscriptions"
	
	subscriptions := IdentitySet new
	


! !

!SubscriptionRegistry methodsFor: 'iterating' stamp: 'IgorStasenko 3/12/2011 17:30'!
subscriptionsOf: aSubscriber do: aBlock
	| copy |
	
	self protected: [ copy := subscriptions copy ].
	
	copy do: 
		[:subscription | 
			subscription subscriber == aSubscriber 
				ifTrue: [ aBlock value: subscription ]	]
! !

Object weakSubclass: #WeakAnnouncementSubscription
	instanceVariableNames: 'list next announcer announcementClass action'
	classVariableNames: 'FinalizationList'
	poolDictionaries: ''
	category: 'Announcements-Core'!

!WeakAnnouncementSubscription commentStamp: 'IgorStasenko 3/12/2011 20:43' prior: 0!
A WeakAnnouncementSubscription is a subscription which is removed automatically when the subscriber is unreferenced.

No support for ephemerons currently prevents this from working for Block actions (blocks hold their receiver, which is the default subscriber strongly). 

To switch between subscription types, use makeStrong/makeWeak on the subscription returned when initially registering with announcer.


Note, that list and next must be first instance variables.!

WeakAnnouncementSubscription weakSubclass: #LegacyWeakSubscription
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Announcements-Core'!

!LegacyWeakSubscription commentStamp: 'IgorStasenko 3/12/2011 21:37' prior: 0!
I am a subclass which work when VM does not supports finalization lists.
I am about 3 times slower when it comes to finalizing items!

!LegacyWeakSubscription methodsFor: 'finalization' stamp: 'IgorStasenko 3/12/2011 21:32'!
finalize
	"When our subscriber dies, we have to remove subscription from announcer."

	announcer removeSubscription: self! !

!LegacyWeakSubscription methodsFor: 'converting' stamp: 'IgorStasenko 3/12/2011 18:32'!
makeStrong

	| sub |
	sub := self subscriber.
	sub ifNil: [ ^ self error: 'Subscriber is nil, cannot make strong subscription' ].
	
	self unregister.

	^ self becomeForward: (AnnouncementSubscription new
		announcer: announcer;
		action: action asMessageSend;
		subscriber: sub;
		announcementClass: announcementClass)
! !

!LegacyWeakSubscription methodsFor: 'converting' stamp: 'IgorStasenko 3/12/2011 17:34'!
makeWeak
	^ self! !

!LegacyWeakSubscription methodsFor: 'finalization' stamp: 'IgorStasenko 3/12/2011 18:19'!
register
	self weakRegistry 
		add: self subscriber executor: self! !

!LegacyWeakSubscription methodsFor: 'accessing' stamp: 'IgorStasenko 3/12/2011 20:01'!
subscriber: anObject

	self subscriber ifNotNil: [ self error: 'subscriber already set' ].
	self basicAt: 1 put: anObject.
	self register! !

!LegacyWeakSubscription methodsFor: 'finalization' stamp: 'IgorStasenko 3/12/2011 17:40'!
unregister

	| sub |
	sub := self subscriber.
	sub ifNotNil: [
		self weakRegistry 
			remove: sub ]
		! !

!LegacyWeakSubscription methodsFor: 'finalization' stamp: 'IgorStasenko 3/12/2011 14:16'!
weakRegistry
	^ WeakRegistry default! !

!WeakAnnouncementSubscription classSide methodsFor: 'finalization support' stamp: 'IgorStasenko 3/12/2011 19:43'!
finalizationList
	
	^ FinalizationList ifNil: [ FinalizationList := WeakFinalizationList new ].! !

!WeakAnnouncementSubscription classSide methodsFor: 'finalization support' stamp: 'IgorStasenko 3/12/2011 21:35'!
finalizeValues
	
	| head |
	
	head := self finalizationList swapWithNil.
	
	[ head notNil ] whileTrue: [ 
		head finalize.
		head := head next
	].! !

!WeakAnnouncementSubscription classSide methodsFor: 'class initialization' stamp: 'IgorStasenko 3/14/2011 13:26'!
initialize
	"self initialize"
	
	
	WeakArray 
		removeWeakDependent: self;
		addWeakDependent: self
		
	"Make sure that it is not added twice."! !

!WeakAnnouncementSubscription classSide methodsFor: 'instance creation' stamp: 'IgorStasenko 3/12/2011 21:32'!
new
	
	^ ((WeakFinalizationList hasNewFinalization
		ifTrue: [ self ] ifFalse: [ LegacyWeakSubscription ] )
			basicNew: 1) initialize! !

!WeakAnnouncementSubscription methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/12/2011 19:57'!
action

	^ action! !

!WeakAnnouncementSubscription methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/12/2011 19:57'!
action: anObject

	action := anObject! !

!WeakAnnouncementSubscription methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/12/2011 19:57'!
announcementClass

	^ announcementClass! !

!WeakAnnouncementSubscription methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/12/2011 19:57'!
announcementClass: anObject

	announcementClass := anObject! !

!WeakAnnouncementSubscription methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/12/2011 19:57'!
announcer

	^ announcer! !

!WeakAnnouncementSubscription methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/12/2011 19:57'!
announcer: anAnnouncer
	announcer := anAnnouncer! !

!WeakAnnouncementSubscription methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/12/2011 19:57'!
deliver: anAnnouncement

	^ (self handles: anAnnouncement class ) ifTrue: [
		action cull: anAnnouncement cull: announcer
		]! !

!WeakAnnouncementSubscription methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/12/2011 21:21'!
finalize

	announcer removeSubscription: self.
	! !

!WeakAnnouncementSubscription methodsFor: 'error handling' stamp: 'IgorStasenko 3/12/2011 19:58'!
handles: anAnnouncementClass

	^ announcementClass handles: anAnnouncementClass! !

!WeakAnnouncementSubscription methodsFor: 'initialize-release' stamp: 'IgorStasenko 3/12/2011 19:48'!
initialize
	list := self class finalizationList! !

!WeakAnnouncementSubscription methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/22/2011 15:45'!
makeStrong

	| sub |
	sub := self subscriber.
	sub ifNil: [ ^ self error: 'Subscriber is nil, cannot make strong subscription' ].
	
	^ announcer replace: self with: 
		(AnnouncementSubscription new
			announcer: announcer;
			action: action asMessageSend;
			subscriber: sub;
			announcementClass: announcementClass)
! !

!WeakAnnouncementSubscription methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/12/2011 19:55'!
makeWeak
	^ self! !

!WeakAnnouncementSubscription methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/12/2011 20:48'!
next
	^ next! !

!WeakAnnouncementSubscription methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/12/2011 19:51'!
subscriber

	^ self basicAt: 1! !

!WeakAnnouncementSubscription methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/12/2011 19:51'!
subscriber: anObject

	self subscriber ifNotNil: [ self error: 'subscriber already set' ].

	self basicAt: 1 put: anObject! !

!WeakAnnouncementSubscription methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/12/2011 19:58'!
valuable: aValuable
	"Used when subscriber should be extracted from valuable object"
	self action:  aValuable.
	self subscriber: aValuable receiver.! !

Object subclass: #WeakSubscriptionBuilder
	instanceVariableNames: 'announcer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Announcements-Core'!

!WeakSubscriptionBuilder commentStamp: 'IgorStasenko 3/12/2011 20:37' prior: 0!
I am a wrapper around an Announcer, used to create weak subscriptions at subscription time.

Use me like this:

anAnnouncer weak subscribe: Announcement send: #foo to: barObject.

I raise an error for block subscriptions, as they require non-existing Ephemeron support to function correctly.!

!WeakSubscriptionBuilder classSide methodsFor: 'instance creation' stamp: 'IgorStasenko 3/12/2011 20:26'!
on: announcer
	^ self new announcer: announcer! !

!WeakSubscriptionBuilder methodsFor: 'initialize-release' stamp: 'IgorStasenko 3/12/2011 20:24'!
announcer: anAnnouncer
	announcer := anAnnouncer ! !

!WeakSubscriptionBuilder methodsFor: 'wrapped protocol' stamp: 'IgorStasenko 3/12/2011 20:24'!
on: anAnnouncementClass do: aValuable
	^ self subscribe: anAnnouncementClass do: aValuable! !

!WeakSubscriptionBuilder methodsFor: 'wrapped protocol' stamp: 'IgorStasenko 3/12/2011 20:25'!
on: anAnnouncementClass send: aSelector to: anObject
	^ self subscribe: anAnnouncementClass send: aSelector to: anObject! !

!WeakSubscriptionBuilder methodsFor: 'errors' stamp: 'IgorStasenko 3/12/2011 20:25'!
requiresEphemeronsSupport
	^ self error: 'Given form of weak subscription requires VM with ephemerons support'! !

!WeakSubscriptionBuilder methodsFor: 'wrapped protocol' stamp: 'IgorStasenko 3/12/2011 20:32'!
subscribe: anAnnouncementClass do: aValuable 
	
	^	announcer basicSubscribe: (
			WeakAnnouncementSubscription new 
				announcer: announcer;
				announcementClass: anAnnouncementClass;
				valuable: aValuable)! !

!WeakSubscriptionBuilder methodsFor: 'wrapped protocol' stamp: 'IgorStasenko 3/12/2011 20:33'!
subscribe: anAnnouncementClass send: aSelector to: anObject
	^ self subscribe: anAnnouncementClass do: (WeakMessageSend receiver: anObject selector: aSelector)! !

!WeakSubscriptionBuilder methodsFor: 'wrapped protocol' stamp: 'IgorStasenko 3/12/2011 20:25'!
weak
	"already weak"
	^ self! !

!WeakSubscriptionBuilder methodsFor: 'wrapped protocol' stamp: 'IgorStasenko 3/12/2011 20:25'!
when: anAnnouncementClass do: aValuable
	^ self subscribe: anAnnouncementClass do: aValuable! !
WeakAnnouncementSubscription initialize!
